# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .generation import Generation as Generation
from .credit_get_response import CreditGetResponse as CreditGetResponse
from .ping_check_response import PingCheckResponse as PingCheckResponse
from .generation_list_params import GenerationListParams as GenerationListParams
from .generation_audio_params import GenerationAudioParams as GenerationAudioParams
from .generation_create_params import GenerationCreateParams as GenerationCreateParams
from .generation_list_response import GenerationListResponse as GenerationListResponse
from .generation_upscale_params import GenerationUpscaleParams as GenerationUpscaleParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal

import httpx

from .image import (
    ImageResource,
    AsyncImageResource,
    ImageResourceWithRawResponse,
    AsyncImageResourceWithRawResponse,
    ImageResourceWithStreamingResponse,
    AsyncImageResourceWithStreamingResponse,
)
from .video import (
    VideoResource,
    AsyncVideoResource,
    VideoResourceWithRawResponse,
    AsyncVideoResourceWithRawResponse,
    VideoResourceWithStreamingResponse,
    AsyncVideoResourceWithStreamingResponse,
)
from ...types import (
    generation_list_params,
    generation_audio_params,
    generation_create_params,
    generation_upscale_params,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .concepts import (
    ConceptsResource,
    AsyncConceptsResource,
    ConceptsResourceWithRawResponse,
    AsyncConceptsResourceWithRawResponse,
    ConceptsResourceWithStreamingResponse,
    AsyncConceptsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.generation import Generation
from ...types.generation_list_response import GenerationListResponse

__all__ = ["GenerationsResource", "AsyncGenerationsResource"]


class GenerationsResource(SyncAPIResource):
    @cached_property
    def concepts(self) -> ConceptsResource:
        return ConceptsResource(self._client)

    @cached_property
    def image(self) -> ImageResource:
        return ImageResource(self._client)

    @cached_property
    def video(self) -> VideoResource:
        return VideoResource(self._client)

    @cached_property
    def with_raw_response(self) -> GenerationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lumalabs/lumaai-python#accessing-raw-response-data-eg-headers
        """
        return GenerationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GenerationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lumalabs/lumaai-python#with_streaming_response
        """
        return GenerationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["ray-2", "ray-flash-2"],
        aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"] | Omit = omit,
        callback_url: str | Omit = omit,
        concepts: Iterable[generation_create_params.Concept] | Omit = omit,
        duration: Union[Literal["5s", "9s"], str] | Omit = omit,
        generation_type: Literal["video"] | Omit = omit,
        keyframes: generation_create_params.Keyframes | Omit = omit,
        loop: bool | Omit = omit,
        prompt: str | Omit = omit,
        resolution: Union[Literal["540p", "720p", "1080p", "4k"], str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Initiate a new generation with the provided prompt

        Args:
          model: The model used for the generation

          aspect_ratio: The aspect ratio of the generation

          callback_url: The callback URL of the generation, a POST request with Generation object will
              be sent to the callback URL when the generation is dreaming, completed, or
              failed

          concepts: The concepts of the generation

          duration: The duration of the generation

          keyframes: The keyframes of the generation

          loop: Whether to loop the video

          prompt: The prompt of the generation

          resolution: The resolution of the generation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/generations/video",
            body=maybe_transform(
                {
                    "model": model,
                    "aspect_ratio": aspect_ratio,
                    "callback_url": callback_url,
                    "concepts": concepts,
                    "duration": duration,
                    "generation_type": generation_type,
                    "keyframes": keyframes,
                    "loop": loop,
                    "prompt": prompt,
                    "resolution": resolution,
                },
                generation_create_params.GenerationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GenerationListResponse:
        """
        Retrieve a list of generations with optional filtering and sorting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/generations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    generation_list_params.GenerationListParams,
                ),
            ),
            cast_to=GenerationListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove a specific generation by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/generations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def audio(
        self,
        id: str,
        *,
        callback_url: str | Omit = omit,
        generation_type: Literal["add_audio"] | Omit = omit,
        negative_prompt: str | Omit = omit,
        prompt: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Add audio to a generation by its ID

        Args:
          callback_url: The callback URL for the audio

          negative_prompt: The negative prompt of the audio

          prompt: The prompt of the audio

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/generations/{id}/audio",
            body=maybe_transform(
                {
                    "callback_url": callback_url,
                    "generation_type": generation_type,
                    "negative_prompt": negative_prompt,
                    "prompt": prompt,
                },
                generation_audio_params.GenerationAudioParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    def get(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Retrieve details of a specific generation by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/generations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    def upscale(
        self,
        id: str,
        *,
        callback_url: str | Omit = omit,
        generation_type: Literal["upscale_video"] | Omit = omit,
        resolution: Union[Literal["540p", "720p", "1080p", "4k"], str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Upscale a generation by its ID

        Args:
          callback_url: The callback URL for the upscale

          resolution: The resolution of the upscale

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/generations/{id}/upscale",
            body=maybe_transform(
                {
                    "callback_url": callback_url,
                    "generation_type": generation_type,
                    "resolution": resolution,
                },
                generation_upscale_params.GenerationUpscaleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )


class AsyncGenerationsResource(AsyncAPIResource):
    @cached_property
    def concepts(self) -> AsyncConceptsResource:
        return AsyncConceptsResource(self._client)

    @cached_property
    def image(self) -> AsyncImageResource:
        return AsyncImageResource(self._client)

    @cached_property
    def video(self) -> AsyncVideoResource:
        return AsyncVideoResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGenerationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lumalabs/lumaai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGenerationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGenerationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lumalabs/lumaai-python#with_streaming_response
        """
        return AsyncGenerationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["ray-2", "ray-flash-2"],
        aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"] | Omit = omit,
        callback_url: str | Omit = omit,
        concepts: Iterable[generation_create_params.Concept] | Omit = omit,
        duration: Union[Literal["5s", "9s"], str] | Omit = omit,
        generation_type: Literal["video"] | Omit = omit,
        keyframes: generation_create_params.Keyframes | Omit = omit,
        loop: bool | Omit = omit,
        prompt: str | Omit = omit,
        resolution: Union[Literal["540p", "720p", "1080p", "4k"], str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Initiate a new generation with the provided prompt

        Args:
          model: The model used for the generation

          aspect_ratio: The aspect ratio of the generation

          callback_url: The callback URL of the generation, a POST request with Generation object will
              be sent to the callback URL when the generation is dreaming, completed, or
              failed

          concepts: The concepts of the generation

          duration: The duration of the generation

          keyframes: The keyframes of the generation

          loop: Whether to loop the video

          prompt: The prompt of the generation

          resolution: The resolution of the generation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/generations/video",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "aspect_ratio": aspect_ratio,
                    "callback_url": callback_url,
                    "concepts": concepts,
                    "duration": duration,
                    "generation_type": generation_type,
                    "keyframes": keyframes,
                    "loop": loop,
                    "prompt": prompt,
                    "resolution": resolution,
                },
                generation_create_params.GenerationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GenerationListResponse:
        """
        Retrieve a list of generations with optional filtering and sorting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/generations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    generation_list_params.GenerationListParams,
                ),
            ),
            cast_to=GenerationListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove a specific generation by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/generations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def audio(
        self,
        id: str,
        *,
        callback_url: str | Omit = omit,
        generation_type: Literal["add_audio"] | Omit = omit,
        negative_prompt: str | Omit = omit,
        prompt: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Add audio to a generation by its ID

        Args:
          callback_url: The callback URL for the audio

          negative_prompt: The negative prompt of the audio

          prompt: The prompt of the audio

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/generations/{id}/audio",
            body=await async_maybe_transform(
                {
                    "callback_url": callback_url,
                    "generation_type": generation_type,
                    "negative_prompt": negative_prompt,
                    "prompt": prompt,
                },
                generation_audio_params.GenerationAudioParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    async def get(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Retrieve details of a specific generation by its ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/generations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    async def upscale(
        self,
        id: str,
        *,
        callback_url: str | Omit = omit,
        generation_type: Literal["upscale_video"] | Omit = omit,
        resolution: Union[Literal["540p", "720p", "1080p", "4k"], str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Generation:
        """
        Upscale a generation by its ID

        Args:
          callback_url: The callback URL for the upscale

          resolution: The resolution of the upscale

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/generations/{id}/upscale",
            body=await async_maybe_transform(
                {
                    "callback_url": callback_url,
                    "generation_type": generation_type,
                    "resolution": resolution,
                },
                generation_upscale_params.GenerationUpscaleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )


class GenerationsResourceWithRawResponse:
    def __init__(self, generations: GenerationsResource) -> None:
        self._generations = generations

        self.create = to_raw_response_wrapper(
            generations.create,
        )
        self.list = to_raw_response_wrapper(
            generations.list,
        )
        self.delete = to_raw_response_wrapper(
            generations.delete,
        )
        self.audio = to_raw_response_wrapper(
            generations.audio,
        )
        self.get = to_raw_response_wrapper(
            generations.get,
        )
        self.upscale = to_raw_response_wrapper(
            generations.upscale,
        )

    @cached_property
    def concepts(self) -> ConceptsResourceWithRawResponse:
        return ConceptsResourceWithRawResponse(self._generations.concepts)

    @cached_property
    def image(self) -> ImageResourceWithRawResponse:
        return ImageResourceWithRawResponse(self._generations.image)

    @cached_property
    def video(self) -> VideoResourceWithRawResponse:
        return VideoResourceWithRawResponse(self._generations.video)


class AsyncGenerationsResourceWithRawResponse:
    def __init__(self, generations: AsyncGenerationsResource) -> None:
        self._generations = generations

        self.create = async_to_raw_response_wrapper(
            generations.create,
        )
        self.list = async_to_raw_response_wrapper(
            generations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            generations.delete,
        )
        self.audio = async_to_raw_response_wrapper(
            generations.audio,
        )
        self.get = async_to_raw_response_wrapper(
            generations.get,
        )
        self.upscale = async_to_raw_response_wrapper(
            generations.upscale,
        )

    @cached_property
    def concepts(self) -> AsyncConceptsResourceWithRawResponse:
        return AsyncConceptsResourceWithRawResponse(self._generations.concepts)

    @cached_property
    def image(self) -> AsyncImageResourceWithRawResponse:
        return AsyncImageResourceWithRawResponse(self._generations.image)

    @cached_property
    def video(self) -> AsyncVideoResourceWithRawResponse:
        return AsyncVideoResourceWithRawResponse(self._generations.video)


class GenerationsResourceWithStreamingResponse:
    def __init__(self, generations: GenerationsResource) -> None:
        self._generations = generations

        self.create = to_streamed_response_wrapper(
            generations.create,
        )
        self.list = to_streamed_response_wrapper(
            generations.list,
        )
        self.delete = to_streamed_response_wrapper(
            generations.delete,
        )
        self.audio = to_streamed_response_wrapper(
            generations.audio,
        )
        self.get = to_streamed_response_wrapper(
            generations.get,
        )
        self.upscale = to_streamed_response_wrapper(
            generations.upscale,
        )

    @cached_property
    def concepts(self) -> ConceptsResourceWithStreamingResponse:
        return ConceptsResourceWithStreamingResponse(self._generations.concepts)

    @cached_property
    def image(self) -> ImageResourceWithStreamingResponse:
        return ImageResourceWithStreamingResponse(self._generations.image)

    @cached_property
    def video(self) -> VideoResourceWithStreamingResponse:
        return VideoResourceWithStreamingResponse(self._generations.video)


class AsyncGenerationsResourceWithStreamingResponse:
    def __init__(self, generations: AsyncGenerationsResource) -> None:
        self._generations = generations

        self.create = async_to_streamed_response_wrapper(
            generations.create,
        )
        self.list = async_to_streamed_response_wrapper(
            generations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            generations.delete,
        )
        self.audio = async_to_streamed_response_wrapper(
            generations.audio,
        )
        self.get = async_to_streamed_response_wrapper(
            generations.get,
        )
        self.upscale = async_to_streamed_response_wrapper(
            generations.upscale,
        )

    @cached_property
    def concepts(self) -> AsyncConceptsResourceWithStreamingResponse:
        return AsyncConceptsResourceWithStreamingResponse(self._generations.concepts)

    @cached_property
    def image(self) -> AsyncImageResourceWithStreamingResponse:
        return AsyncImageResourceWithStreamingResponse(self._generations.image)

    @cached_property
    def video(self) -> AsyncVideoResourceWithStreamingResponse:
        return AsyncVideoResourceWithStreamingResponse(self._generations.video)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VideoModifyParams", "Media", "FirstFrame"]


class VideoModifyParams(TypedDict, total=False):
    generation_type: Required[Literal["modify_video"]]

    media: Required[Media]
    """The image entity object"""

    mode: Required[
        Literal[
            "adhere_1",
            "adhere_2",
            "adhere_3",
            "flex_1",
            "flex_2",
            "flex_3",
            "reimagine_1",
            "reimagine_2",
            "reimagine_3",
        ]
    ]
    """The mode of the modify video"""

    model: Required[Literal["ray-2", "ray-flash-2"]]
    """The model used for the modify video"""

    callback_url: str
    """
    The callback URL of the generation, a POST request with Generation object will
    be sent to the callback URL when the generation is dreaming, completed, or
    failed
    """

    first_frame: FirstFrame
    """The image entity object"""

    prompt: str
    """The prompt of the generation"""


class Media(TypedDict, total=False):
    """The image entity object"""

    url: Required[str]
    """The URL of the media"""


class FirstFrame(TypedDict, total=False):
    """The image entity object"""

    url: Required[str]
    """The URL of the media"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import LumaAIError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import ping, credits, generations
    from .resources.ping import PingResource, AsyncPingResource
    from .resources.credits import CreditsResource, AsyncCreditsResource
    from .resources.generations.generations import GenerationsResource, AsyncGenerationsResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "LumaAI", "AsyncLumaAI", "Client", "AsyncClient"]


class LumaAI(SyncAPIClient):
    # client options
    auth_token: str

    def __init__(
        self,
        *,
        auth_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous LumaAI client instance.

        This automatically infers the `auth_token` argument from the `LUMAAI_API_KEY` environment variable if it is not provided.
        """
        if auth_token is None:
            auth_token = os.environ.get("LUMAAI_API_KEY")
        if auth_token is None:
            raise LumaAIError(
                "The auth_token client option must be set either by passing auth_token to the client or by setting the LUMAAI_API_KEY environment variable"
            )
        self.auth_token = auth_token

        if base_url is None:
            base_url = os.environ.get("LUMAAI_BASE_URL")
        if base_url is None:
            base_url = f"https://api.lumalabs.ai/dream-machine/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def generations(self) -> GenerationsResource:
        from .resources.generations import GenerationsResource

        return GenerationsResource(self)

    @cached_property
    def ping(self) -> PingResource:
        from .resources.ping import PingResource

        return PingResource(self)

    @cached_property
    def credits(self) -> CreditsResource:
        from .resources.credits import CreditsResource

        return CreditsResource(self)

    @cached_property
    def with_raw_response(self) -> LumaAIWithRawResponse:
        return LumaAIWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LumaAIWithStreamedResponse:
        return LumaAIWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        auth_token = self.auth_token
        return {"Authorization": f"Bearer {auth_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        auth_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            auth_token=auth_token or self.auth_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncLumaAI(AsyncAPIClient):
    # client options
    auth_token: str

    def __init__(
        self,
        *,
        auth_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncLumaAI client instance.

        This automatically infers the `auth_token` argument from the `LUMAAI_API_KEY` environment variable if it is not provided.
        """
        if auth_token is None:
            auth_token = os.environ.get("LUMAAI_API_KEY")
        if auth_token is None:
            raise LumaAIError(
                "The auth_token client option must be set either by passing auth_token to the client or by setting the LUMAAI_API_KEY environment variable"
            )
        self.auth_token = auth_token

        if base_url is None:
            base_url = os.environ.get("LUMAAI_BASE_URL")
        if base_url is None:
            base_url = f"https://api.lumalabs.ai/dream-machine/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def generations(self) -> AsyncGenerationsResource:
        from .resources.generations import AsyncGenerationsResource

        return AsyncGenerationsResource(self)

    @cached_property
    def ping(self) -> AsyncPingResource:
        from .resources.ping import AsyncPingResource

        return AsyncPingResource(self)

    @cached_property
    def credits(self) -> AsyncCreditsResource:
        from .resources.credits import AsyncCreditsResource

        return AsyncCreditsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncLumaAIWithRawResponse:
        return AsyncLumaAIWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLumaAIWithStreamedResponse:
        return AsyncLumaAIWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        auth_token = self.auth_token
        return {"Authorization": f"Bearer {auth_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        auth_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            auth_token=auth_token or self.auth_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class LumaAIWithRawResponse:
    _client: LumaAI

    def __init__(self, client: LumaAI) -> None:
        self._client = client

    @cached_property
    def generations(self) -> generations.GenerationsResourceWithRawResponse:
        from .resources.generations import GenerationsResourceWithRawResponse

        return GenerationsResourceWithRawResponse(self._client.generations)

    @cached_property
    def ping(self) -> ping.PingResourceWithRawResponse:
        from .resources.ping import PingResourceWithRawResponse

        return PingResourceWithRawResponse(self._client.ping)

    @cached_property
    def credits(self) -> credits.CreditsResourceWithRawResponse:
        from .resources.credits import CreditsResourceWithRawResponse

        return CreditsResourceWithRawResponse(self._client.credits)


class AsyncLumaAIWithRawResponse:
    _client: AsyncLumaAI

    def __init__(self, client: AsyncLumaAI) -> None:
        self._client = client

    @cached_property
    def generations(self) -> generations.AsyncGenerationsResourceWithRawResponse:
        from .resources.generations import AsyncGenerationsResourceWithRawResponse

        return AsyncGenerationsResourceWithRawResponse(self._client.generations)

    @cached_property
    def ping(self) -> ping.AsyncPingResourceWithRawResponse:
        from .resources.ping import AsyncPingResourceWithRawResponse

        return AsyncPingResourceWithRawResponse(self._client.ping)

    @cached_property
    def credits(self) -> credits.AsyncCreditsResourceWithRawResponse:
        from .resources.credits import AsyncCreditsResourceWithRawResponse

        return AsyncCreditsResourceWithRawResponse(self._client.credits)


class LumaAIWithStreamedResponse:
    _client: LumaAI

    def __init__(self, client: LumaAI) -> None:
        self._client = client

    @cached_property
    def generations(self) -> generations.GenerationsResourceWithStreamingResponse:
        from .resources.generations import GenerationsResourceWithStreamingResponse

        return GenerationsResourceWithStreamingResponse(self._client.generations)

    @cached_property
    def ping(self) -> ping.PingResourceWithStreamingResponse:
        from .resources.ping import PingResourceWithStreamingResponse

        return PingResourceWithStreamingResponse(self._client.ping)

    @cached_property
    def credits(self) -> credits.CreditsResourceWithStreamingResponse:
        from .resources.credits import CreditsResourceWithStreamingResponse

        return CreditsResourceWithStreamingResponse(self._client.credits)


class AsyncLumaAIWithStreamedResponse:
    _client: AsyncLumaAI

    def __init__(self, client: AsyncLumaAI) -> None:
        self._client = client

    @cached_property
    def generations(self) -> generations.AsyncGenerationsResourceWithStreamingResponse:
        from .resources.generations import AsyncGenerationsResourceWithStreamingResponse

        return AsyncGenerationsResourceWithStreamingResponse(self._client.generations)

    @cached_property
    def ping(self) -> ping.AsyncPingResourceWithStreamingResponse:
        from .resources.ping import AsyncPingResourceWithStreamingResponse

        return AsyncPingResourceWithStreamingResponse(self._client.ping)

    @cached_property
    def credits(self) -> credits.AsyncCreditsResourceWithStreamingResponse:
        from .resources.credits import AsyncCreditsResourceWithStreamingResponse

        return AsyncCreditsResourceWithStreamingResponse(self._client.credits)


Client = LumaAI

AsyncClient = AsyncLumaAI



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.credit_get_response import CreditGetResponse

__all__ = ["CreditsResource", "AsyncCreditsResource"]


class CreditsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CreditsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lumalabs/lumaai-python#accessing-raw-response-data-eg-headers
        """
        return CreditsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreditsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lumalabs/lumaai-python#with_streaming_response
        """
        return CreditsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGetResponse:
        """Get the credits information for the api user"""
        return self._get(
            "/credits",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGetResponse,
        )


class AsyncCreditsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCreditsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lumalabs/lumaai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCreditsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreditsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lumalabs/lumaai-python#with_streaming_response
        """
        return AsyncCreditsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditGetResponse:
        """Get the credits information for the api user"""
        return await self._get(
            "/credits",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditGetResponse,
        )


class CreditsResourceWithRawResponse:
    def __init__(self, credits: CreditsResource) -> None:
        self._credits = credits

        self.get = to_raw_response_wrapper(
            credits.get,
        )


class AsyncCreditsResourceWithRawResponse:
    def __init__(self, credits: AsyncCreditsResource) -> None:
        self._credits = credits

        self.get = async_to_raw_response_wrapper(
            credits.get,
        )


class CreditsResourceWithStreamingResponse:
    def __init__(self, credits: CreditsResource) -> None:
        self._credits = credits

        self.get = to_streamed_response_wrapper(
            credits.get,
        )


class AsyncCreditsResourceWithStreamingResponse:
    def __init__(self, credits: AsyncCreditsResource) -> None:
        self._credits = credits

        self.get = async_to_streamed_response_wrapper(
            credits.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ImageCreateParams", "CharacterRef", "CharacterRefIdentity0", "ImageRef", "ModifyImageRef", "StyleRef"]


class ImageCreateParams(TypedDict, total=False):
    model: Required[Literal["photon-1", "photon-flash-1"]]
    """The model used for the generation"""

    aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"]
    """The aspect ratio of the generation"""

    callback_url: str
    """The callback URL for the generation"""

    character_ref: CharacterRef

    format: Literal["jpg", "png"]
    """The format of the image"""

    generation_type: Literal["image"]

    image_ref: Iterable[ImageRef]

    modify_image_ref: ModifyImageRef
    """The modify image reference object"""

    prompt: str
    """The prompt of the generation"""

    style_ref: Iterable[StyleRef]

    sync: bool
    """Create image in synchronous mode and return complated image"""

    sync_timeout: float
    """The timeout for the synchronous image generation"""


class CharacterRefIdentity0(TypedDict, total=False):
    """The image identity object"""

    images: SequenceNotStr[str]
    """The URLs of the image identity"""


class CharacterRef(TypedDict, total=False):
    identity0: CharacterRefIdentity0
    """The image identity object"""


class ImageRef(TypedDict, total=False):
    """The image reference object"""

    url: str
    """The URL of the image reference"""

    weight: float
    """The weight of the image reference"""


class ModifyImageRef(TypedDict, total=False):
    """The modify image reference object"""

    url: str
    """The URL of the image reference"""

    weight: float
    """The weight of the modify image reference"""


class StyleRef(TypedDict, total=False):
    """The image reference object"""

    url: str
    """The URL of the image reference"""

    weight: float
    """The weight of the image reference"""
